/****************************************************************/
/*           S A S   G L O B A L   F O R U M   2 0 1 6          */
/*                                                              */
/*   TITLE: Highly Customized Graphs Using ODS Graphics         */
/* PRODUCT: BASE and STAT                                       */
/*                                                              */
/****************************************************************/

ods graphics on;

proc cluster data=sashelp.class method=ward pseudo;
   ods select dendrogram;
   id name;
run;

ods document name=MyDoc (write);
proc cluster data=sashelp.class method=ward pseudo;
   ods select dendrogram;
   id name;
run;
ods document close;

proc document name=MyDoc;
   list / levels=all;
quit;

proc document name=MyDoc;
   replay \Cluster#1\Dendrogram#1;
quit;

proc document name=MyDoc;
   ods output dynamics=dynamics;
   obdynam \Cluster#1\Dendrogram#1;
quit;

proc print noobs data=dynamics;
run;

proc document name=MyDoc;
   replay \Cluster#1\Dendrogram#1 / dynamdata=dynamics;
quit;

data dynamics2;
   set dynamics;
   if label1 = 'DH' then cvalue1 = '400PX';
   if label1 = 'DW' then cvalue1 = '400PX';
run;

proc document name=MyDoc;
   replay \Cluster#1\Dendrogram#1 / dynamdata=dynamics2;
quit;

ods trace on;

proc template;
   source Stat.Cluster.Graphics.Dendrogram;
quit;

proc template;
   define statgraph Stat.Cluster.Graphics.Dendrogram;
      notes "Dendrogram";
      dynamic dh dw orient xlabel ylabel hvar xr yr _byline_ _bytitle_
         _byfootnote_;
      begingraph / designheight=DH designwidth=DW;
         entrytitle "Cluster Analysis of the Class Data";
         layout overlay / xaxisopts=(label=' ' reverse=XR) yaxisopts=(label=
            YLABEL reverse=YR discreteopts=(tickvaluefitpolicy=none));
            dendrogram nodeid=NAME parentid=PARENT clusterheight=HVAR /
               orient=vertical;
         endlayout;
         if (_BYTITLE_)
            entrytitle _BYLINE_ / textattrs=GRAPHVALUETEXT;
         else
            if (_BYFOOTNOTE_)
               entryfootnote halign=left _BYLINE_;
            endif;
         endif;
      endgraph;
   end;
quit;

data dynamics2;
   set dynamics;
   if label1 = 'DH' then cvalue1 = '400PX';
   if label1 = 'DW' then cvalue1 = '600PX';
run;

proc document name=MyDoc;
   replay \Cluster#1\Dendrogram#1 / dynamdata=dynamics2;
quit;

proc document name=MyDoc;
   replay \Cluster#1\Dendrogram#1 / dynamdata=dynamics2;
quit;

proc cluster data=sashelp.class method=ward pseudo
             plots=dendrogram(vertical setheight=400 setwidth=600);
   ods select dendrogram;
   id name;
run;

ods listing close;
ods listing;
proc template;
   delete Stat.Cluster.Graphics.Dendrogram / store=sasuser.templat;
run;

ods document name=MyDoc (write);
proc reg data=sashelp.class;
   model weight = height age / clb;
quit;
ods document close;

proc document name=MyDoc;
   list / levels=all;
quit;

proc document name=MyDoc;
   ods output dynamics=dynamics;
   %let p = \Work.Mydoc\Reg#1\MODEL1#1\ObswiseStats#1\Weight#1;
   obdynam &p\DiagnosticPlots#1\DiagnosticsPanel#1;
quit;

proc print noobs data=dynamics;
   where trim(label1) in ('_NPARM' '_TOTFREQ' '_NOBS' '_SHOWSTATS'
         '_NSTATSCOLS' '_SHOWNOBS' '_SHOWTOTFREQ' '_SHOWNPARM' '_SHOWEDF'
         '_EDF' '_SHOWMSE' '_MSE' '_SHOWRSQUARE' '_RSQUARE' '_SHOWADJRSQ'
         '_ADJRSQ' '_SHOWSSE' '_SSE' '_SHOWDEPMEAN' '_DEPMEAN' '_SHOWCV'
         '_CV' '_SHOWAIC' '_AIC' '_SHOWBIC' '_BIC' '_DEPLABEL');
run;

data dynamics2;
   set dynamics;
   if label1 = '_SHOWADJRSQ' then nvalue1 = 0;
   if label1 = '_SHOWSBC'    then nvalue1 = 1;
run;

proc document name=MyDoc;
   replay &p\DiagnosticPlots#1\DiagnosticsPanel#1 / dynamdata=dynamics2;
quit;

proc reg data=sashelp.class
         plots=diagnostics(stats=(nobs nparm edf mse rsquare sbc));
   model weight = height age / clb;
quit;

data bivnormal;
   do i = 1 to 1000;
      z1 =      rannor(17151377);
      x  = 3*z1+rannor(17151377);
      y  = 3*z1+rannor(17151377);
      output;
   end;
run;

ods document name=MyDoc(write);
proc kde data=bivnormal;
   bivar x y / plots=(contour surface);
run;
ods document close;

proc document name=MyDoc;
   list / levels=all;
quit;

proc document name=MyDoc;
   ods output dynamics=dynamics;
   obdynam \KDE#1\Bivar1#1\x_y#1\ContourPlot#1;
quit;

data dynamics2;
   length cvalue1 $ 13;
   set dynamics;
   if label1 = '_VAR1NAME' then cvalue1 = 'an X Variable';
   if label1 = '_VAR2NAME' then cvalue1 = 'a Y Variable';
run;

proc document name=MyDoc;
   replay \KDE#1\Bivar1#1\x_y#1\ContourPlot#1 / dynamdata=dynamics2;
quit;

proc template; delete Stat.REG.Graphics.DiagnosticsPanel; quit;
ods graphics on;
proc reg data=sashelp.class;
   ods select diagnosticspanel;
   ods output diagnosticspanel=dp;
   model weight = height;
quit;

proc sgrender data=dp template=Stat.REG.Graphics.DiagnosticsPanel;
run;

ods document name=MyDoc (write);
proc reg data=sashelp.class;
   ods select diagnosticspanel;
   ods output diagnosticspanel=dp;
   model weight = height;
quit;
ods document close;

proc document name=MyDoc;
   list / levels=all;
quit;

proc document name=MyDoc;
   ods output dynamics=outdynam;
   obdynam \Reg#1\MODEL1#1\ObswiseStats#1\Weight#1\DiagnosticPlots#1\DiagnosticsPanel#1;
quit;

data anno;
   length Label $ 40;
   Function = 'Text';     Label     = 'Saturday, July 25, 2015';
   Width    = 100;        x1        = 99;   y1 = .1;
   Anchor   = 'Right ';   TextColor = 'Red';
   output;

   Label = 'Confidential - Do Not Distribute';
   Width = 150;           x1        = 50;   y1     =  50;   Anchor = 'Center';
   Transparency = 0.8;    TextSize  = 40;   Rotate = -45;
   output;
run;

proc print noobs;
run;

proc template;
   source Stat.REG.Graphics.DiagnosticsPanel / file='temp.tmp';
quit;

data _null_;
   infile 'temp.tmp';
   input;
   if _n_ = 1 then call execute('proc template;');
   call execute(_infile_);
   if _infile_ =: '   BeginGraph' then bg + 1;
   if bg and index(_infile_, ';') then do;
      bg = 0;
      call execute('annotate;');
   end;
run;

data _null_;
   set outdynam(where=(label1 ne '___NOBS___')) end=eof;
   if _n_ = 1 then do;
      call execute('proc sgrender data=dp sganno=anno');
      call execute('template=Stat.REG.Graphics.DiagnosticsPanel;');
      call execute('dynamic');
   end;
   if label1 = '_SHOWEDF' then cvalue1 = '0';
   if cvalue1 ne ' ' then
      call execute(catx(' ', label1, '=',
                   ifc(n(nvalue1), cvalue1, quote(trim(cvalue1)))));
   if eof then call execute('; run;');
run;

data _null_;
   infile 'temp.tmp';
   input;
   if _n_ = 1 then call execute('proc template;');

   i = index(_infile_, 'BEST6.');
   if i and (index(_infile_, '_ADJRSQ') or index(_infile_, '_RSQUARE'))
      then substr(_infile_, i, 6) = '6.4';

   call execute(_infile_);
   if _infile_ =: '   BeginGraph' then bg + 1;
   if bg and index(_infile_, ';') then do;
      bg = 0;
      call execute('annotate;');
   end;
run;

data _null_;
   set outdynam(where=(label1 ne '___NOBS___')) end=eof;
   if _n_ = 1 then do;
      call execute('proc sgrender data=dp sganno=anno');
      call execute('template=Stat.REG.Graphics.DiagnosticsPanel;');
      call execute('dynamic');
   end;
   if label1 = '_SHOWEDF' then cvalue1 = '0';
   if cvalue1 ne ' ' then
      call execute(catx(' ', label1, '=',
                   ifc(n(nvalue1), cvalue1, quote(trim(cvalue1)))));
   if eof then call execute('; run;');
run;

proc template;
   delete Stat.REG.Graphics.DiagnosticsPanel;
quit;

ods document name=MyDoc (write);
proc reg data=sashelp.class;
   ods select diagnosticspanel;
   ods output diagnosticspanel=dp;
   model weight = height;
quit;
ods document close;

data anno;
   length Label $ 40;
   Function = 'Text';     Label     = 'Saturday, July 25, 2015';
   Width    = 100;        x1        = 99;   y1 = .1;
   Anchor   = 'Right';    TextColor = 'Red';
   output;

   Label = 'Confidential - Do Not Distribute';
   Width = 150;           x1        = 50;   y1     =  50;   Anchor = 'Center';
   Transparency = 0.8;    TextSize  = 40;   Rotate = -45;
   output;
run;

%macro procanno(data=, template=, anno=anno, document=mydoc);
   proc document name=&document;
      ods exclude properties;
      ods output properties=__p(where=(type='Graph'));
      list / levels=all;
   quit;

   data _null_;
      set __p;
      call execute("proc document name=&document;");
      call execute("ods exclude dynamics;");
      call execute("ods output dynamics=__outdynam;");
      call execute(catx(' ', "obdynam", path, ';'));
   run;

   proc template;
      source &template / file='temp.tmp';
   quit;

   data _null_;
      infile 'temp.tmp';
      input;
      if _n_ = 1 then call execute('proc template;');
      call execute(_infile_);
      if _infile_ =: '   BeginGraph' then bg + 1;
      if bg and index(_infile_, ';') then do;
         bg = 0;
         call execute('annotate;');
      end;
   run;

   data _null_;
      set __outdynam(where=(label1 ne '___NOBS___')) end=eof;
      if _n_ = 1 then do;
         call execute("proc sgrender data=&data sganno=&anno");
         call execute("template=&template;");
         call execute('dynamic');
      end;
      if cvalue1 ne ' ' then
         call execute(catx(' ', label1, '=',
                      ifc(n(nvalue1), cvalue1, quote(trim(cvalue1)))));
      if eof then call execute('; run;');
   run;

   proc template;
      delete &template;
   quit;
%mend;

%procanno(data=dp, template=Stat.REG.Graphics.DiagnosticsPanel)

ods graphics on;
proc glmselect data=sashelp.baseball plots=coefficients;
   class league division;
   model logSalary = nAtBat nHits nHome nRuns nRBI nBB yrMajor|yrMajor
                     crAtBat|crAtBat crHits|crHits crHome|crHome crRuns|crRuns
                     crRbi|crRbi crBB|crBB league division nOuts nAssts nError /
                     selection=forward(stop=AICC CHOOSE=SBC);
run;

ods document name=MyDoc (write);
proc glmselect data=sashelp.baseball plots=coefficients;
   ods select CoefficientPanel;
   ods output CoefficientPanel=cp;
   class league division;
   model logSalary = nAtBat nHits nHome nRuns nRBI nBB yrMajor|yrMajor
                     crAtBat|crAtBat crHits|crHits crHome|crHome crRuns|crRuns
                     crRbi|crRbi crBB|crBB league division nOuts nAssts nError /
                     selection=forward(stop=AICC CHOOSE=SBC);
run;
ods document close;

data labelthese(keep=par);
   set cp end=eof;
   retain f1-f3 1;
   if f1 and steplabel =: '1+' then do; f1 = 0; link s; end;
   if f2 and steplabel =: '2+' then do; f2 = 0; link s; end;
   if f3 and steplabel =: '3+' then do; f3 = 0; link s; end;
   if eof then call symputx('_step', step);
   return;
s: par = substr(steplabel, 3);
   output;
   return;
run;

proc print noobs;
run;

data cp2;
   set cp;
   match = 0;
   if step ne &_step then return;
   do i = 1 to ntolabel;
      set labelthese point=i nobs=ntolabel;
      match + (par = parameter);
      end;
   if not match then parameter = ' ';
   if nmiss(rhslabelYvalue) then rhslabelYvalue = StandardizedEst;
run;

%macro tweak;
   if index(_infile_, 'datalabel=PARAMETER') then
      _infile_ = tranwrd(_infile_, 'datalabel',
                          'markercharacterposition=right markercharacter');
   if index(_infile_, 'curvelabel="Selected Step"') then
      _infile_ = tranwrd(_infile_, 'curvelabel="Selected Step"', ' ');
%mend;

data anno;
   length ID $ 3 Function $ 9 Label $ 40;
   retain x1Space y1Space x2Space y2Space 'DataPercent' Direction 'In';
   length Anchor $ 10 xC1 xC2 $ 20;
   retain Scale 1e-12 Width 100 WidthUnit 'Data' CornerRadius 0.8
          TextSize 7 TextWeight 'Bold'
          LineThickness 1.2 DiscreteOffset -0.3 LineColor 'Green';

   ID       = 'LO1';            Function  = 'Text';
   Anchor   = 'Right';          TextColor = 'Green';
   x1       = 55;               y1        = 94;
   Label    = 'Coefficients for the Selected Model';                output;

   Function = 'Line';           x1        = .;
   x1Space  = 'DataValue';      x2Space   = x1Space;
   xc1      = '9+CrBB';         xc2       = '8+CrRuns*CrRuns';
   y1       = 94;               y2        = 94;                     output;

   Function = 'Rectangle';      y1Space   = 'WallPercent';
   Anchor   = 'BottomLeft';     y1        = 10;
   Height   = 80;               Width     = 0.6;                    output;

   ID       = 'LO3';            Width     = 100;
   Function = 'Text ';          Label     = 'Selected Value';
   x1Space  = 'DataPercent';    y1Space   = x1Space;
   Anchor   = 'Left';           TextColor = 'Blue';
   x1       = 86;               y1        = 84;                     output;

   Function = 'Arrow';          LineColor = 'Blue';
   x1Space  = 'DataValue';      x2Space   = x1Space;
   xc1      = '9+CrBB';         xc2       = '12+CrHits*CrHits';
   y1       = 4;                y2        = 83;
   DiscreteOffset = .1;         x1        = .;                      output;
run;

%macro procannoadv(data=, template=, anno=anno, document=mydoc, adjust=,
                   overallanno=1);

   proc document name=&document;
      ods exclude properties;
      ods output properties=__p(where=(type='Graph'));
      list / levels=all;
   quit;

   data _null_;
      set __p;
      call execute("proc document name=&document;");
      call execute("ods exclude dynamics;");
      call execute("ods output dynamics=__outdynam;");
      call execute(catx(' ', "obdynam", path, ';'));
   run;

   proc template;
      source &template / file='temp.tmp';
   quit;

   data _null_;
      infile 'temp.tmp';
      input;
      if _n_ = 1 then call execute('proc template;');
      %if &adjust ne %then %do; %&adjust %end;
      call execute(_infile_);
      if &overallanno and _infile_ =:     '   BeginGraph' then bg + 1;
      else if not &overallanno and index(_infile_, '   layout overlay')
         then lo + 1;
      if bg and index(_infile_, ';') then do;
         bg = 0;
         call execute('annotate;');
      end;
      if lo and index(_infile_, ';') then do;
         lo = 0;
         lonum + 1;
         call execute(catt('annotate / id="LO', lonum, '";'));
      end;
   run;

   data _null_;
      set __outdynam(where=(label1 ne '___NOBS___')) end=eof;
      if _n_ = 1 then do;
         call execute("proc sgrender data=&data");
         if symget('anno') ne ' ' then call execute("sganno=&anno");
         call execute("template=&template;");
         call execute('dynamic');
      end;
      if cvalue1 ne ' ' then
         call execute(catx(' ', label1, '=',
                      ifc(n(nvalue1), cvalue1, quote(trim(cvalue1)))));
      if eof then call execute('; run;');
   run;

   proc template;
      delete &template;
   quit;
%mend;

%procannoadv(data=cp2, template=Stat.GLMSELECT.Graphics.CoefficientPanel,
             adjust=tweak, overallanno=0)

data labelthese(keep=parameter rename=(parameter=par));
   set cp end=eof;
   if eof then call symputx('_step', step);
   if step = 9 and (StandardizedEst gt 1 or StandardizedEst lt -1);
run;

data cp2;
   set cp;
   match = 0;
   if step ne &_step then return;
   do i = 1 to ntolabel;
      set labelthese point=i nobs=ntolabel;
      match + (par = parameter);
      end;
   if not match then parameter = ' ';
   if nmiss(rhslabelYvalue) then rhslabelYvalue = StandardizedEst;
run;

%macro tweak;
   if index(_infile_, 'datalabel=PARAMETER') then
      _infile_ = tranwrd(_infile_, 'datalabel',
                          'markercharacterposition=right markercharacter');
%mend;

%procannoadv(data=cp2, template=Stat.GLMSELECT.Graphics.CoefficientPanel,
             anno=, adjust=tweak)

